/************************************************************************
 *  RS232.C								*
 *			RS232-Funktionen fr MON51			*
 *			==========================			*
 *									*
 *  17.06.93	Bodo Wenzel	V1.0, Erstellung			*
 ************************************************************************/

#pragma code symbols small debug

/* === Beschreibung =================================================== */

/* Die folgende Tabelle listet fr einige Quarzfrequenzen und Baudraten
   die theoretisch erzielbaren Baudraten auf. Dabei sind nicht mehr
   praktikable Baudraten in Klammern gesetzt.
   Nach einigen Experimenten wurden noch ein paar Eintrge mehr als nicht
   mglich erkannt. Trotzdem bleiben gengend Baudraten zur praktischen
   Nutzung brig. Die Faustregel lautet: "Bei hohen Quarzfrequenzen hohe
   Baudraten (z. B. 4800), bei kleinen Quarzfrequenzen kleine Baudraten
   (z. B. 1200) probieren".

   Die benutzten Formeln (siehe 8051-Beschreibung, Serial Mode 1):
   TH1      = 256 - (Quarzfrequenz / 12 / 16 / Baudrate)
   Baudrate = Quarzfrequenz / 12 / 16 / (256-TH1)

Baud	300	600	1200	2400	3600	4800	9600	19200
-10%	270	540	1080	2160	3240	4320	8640	17280
+10%	330	660	1320	2640	3960	5280	10500	21120

	Quarz=2457600 MHz:
TH1=	213	235	245	251	252	253	255	255
Baud=	298	609	1164	(2560)	(3200)	(4267)	(12800)	(12800)
	Quarz=3579000 MHz:
TH1=	194	225	240	248	251	252	254	255
Baud=	301	601	1165	2330	3728	4660	9320	(18641)
	Quarz=3686400 MHz:
TH1=	192	224	240	248	251	252	254	255
Baud=	300	600	1200	2400	(3840)	4800	9600	(19200)
	Quarz=4915200 MHz:
TH1=	171	213	235	245	249	251	253	255
Baud=	301	595	1219	2327	3657	(5120)	(8533)	(25600)
	Quarz=5000000 MHz:
TH1=	169	213	234	245	249	251	253	255
Baud=	299	606	1184	2367	3720	(5208)	(8681)	(26042)
	Quarz=6000000 MHz:
TH1=	152	204	230	243	247	249	253	254
Baud=	301	601	1202	2404	3472	(4464)	(10417)	(15625)
	Quarz=8000000 MHz:
TH1=	117	187	221	239	244	247	252	254
Baud=	300	604	1191	2451	3472	4630	(10417)	(20833)
	Quarz=11059200 MHz:
TH1=	64	160	208	232	240	244	250	253
Baud=	300	600	1200	2400	3600	4800	9600	19200
	Quarz=12000000 MHz:
TH1=	48	152	204	230	239	243	249	253
Baud=	301	601	1202	2404	3677	4808	(8929)	(20833)
	Quarz=14000000 MHz:
TH1=	13	134	195	226	236	241	248	252
Baud=	300	598	1195	2431	3646	4861	(9115)	(18229)
	Quarz=16000000 MHz:
TH1=	-21	117	187	221	233	239	247	252
Baud=	---	600	1208	2381	3623	4902	9259	(20833)
	Quarz=18000000 MHz:
TH1=	-56	100	178	217	230	236	246	251
Baud=	---	601	1202	2404	3606	4688	9375	18750

   Die Ermittlung der Baudrate luft ber den Timer 1, der schlielich
   auch zur Erzeugung der Baudrate benutzt wird. Zunchst wird auf ein
   Startbit gewartet, dessen Lnge dann gemessen wird. Unter der Voraus-
   setzung, da das Bit 0 des bertragenen Datenbytes "1" ist, kann der
   ausgelesene Timerwert nach Teilung durch 16 (der 8051-interne
   Overscanfaktor) direkt als Teilerwert verwendet werden. Nach der
   Messung des Startbits wird der Rest des Datenbytes ausgelassen.

   Der angeschlossene Host sendet also zwei Bytes: ein beliebiges Byte
   mit Bit 0 = "1" und danach ein "Carriage-Return" (0x0D), das als
   Prfzeichen dient. Erst danach wird die Baudrate als erkannt
   akzeptiert.
*/

/* === Einfgedateien ================================================= */

#include <reg51.h>
#include <stdio.h>

#include "utility.h"
#include "rs232.h"

/* === Funktionen ===================================================== */

uint rs232_init(void) {
/*-------------------*/
uint	t;

SCON=0x58;			/* 8-N-1, Empfang freigeben		*/
PCON|=0x80;			/* doppelte Baudrate			*/

for (;;) {
  TR1=FALSE;
  TMOD&=0x0F;
  TMOD|=0x10;
  TH1=0;
  TL1=0;			/* Timer 1 stoppen und lschen		*/
  while (RXD);
  TR1=TRUE;			/* Bei fallender Flanke starten		*/
  while (!RXD);
  TR1=FALSE;
  t=((uint)TH1<<8) | TL1;	/* Bei steigender Flanke messen		*/

  if (t>0xFF7)
    continue;			/* Zu langer Lowpuls			*/

  TH1=(-(int)t*8)>>8;
  TL1=(uchar)(-(int)t*8);
  TF1=FALSE;
  TR1=TRUE;
  while (!TF1);
  TR1=FALSE;			/* Rest des Datenbytes abwarten		*/

  TMOD&=0x0F;
  TMOD|=0x20;
  TH1=-(uchar)((t+8)/16);
  TR1=TRUE;			/* als Baudraten-Timer setzen		*/

  RI=FALSE;
  while (!RI);
  if (SBUF=='\r')
    break;			/* auf CR warten			*/
  }

RI=FALSE;			/* Empfnger bereit			*/
TI=TRUE;			/* Sender bereit			*/

return(12*t);			/* Anzahl Taktzyklen zurckgeben	*/
}

/* --- Standard-Ausgabe (weniger Speicher) ---------------------------- */

void rs232_puts(char *s) {
/*----------------------*/
while (*s)
  putchar(*s++);
}

void rs232_putd(uint v) {
/*---------------------*/
static	uint	code	div[4]={ 10000, 1000, 100, 10 };
	bit		o;
	uchar		i;

o=FALSE;
for (i=0; i<4; i++) {
  if (v>=div[i] || o) {
    putchar('0'+v/div[i]);
    v%=div[i];
    o=TRUE;
    }
  }
putchar('0'+v);
}

void rs232_putn(uchar v) {
/*----------------------*/
v&=0xF;
if (v>=0xA)
  putchar(v+'A'-0xA);
else
  putchar(v+'0');
}

void rs232_putx(uint v,bit ad) {
/*----------------------------*/
if (v>0xFF || ad) {
  rs232_putn((uchar)(v>>12));
  rs232_putn((uchar)(v>>8));
  }
rs232_putn((uchar)(v>>4));
rs232_putn((uchar)v);
}

/* === Ende =========================================================== */
